from __future__ import print_function

import datetime

def __sanitize_label_for_version(label):
    # Strings in version fields must be sanitized to the intersection of allowable things in rpm and dpkg
    # That means no dashes, underscores, forward/back slashes, and colons
    # for note: rpm allows only the following separators: period (.), underscore (_), plus (+), and tilde (~)
    # for note: dpkg allows only the following separators: period (.), dash (-), plus (+), and tilde (~)
    # The intersection of the two means we can only use periods, pluses, and tildes
    # The current rules are as follows:
    # * underscore/dash -> eliminate
    # * colons -> convert to periods
    # * slashes -> convert to pluses
    # * Everything is converted to lowercase
    return label.replace('-', '').replace('_', '').replace(':', '.').replace('/', '+').replace('\\', '+').lower()

def generate_snapshot_version(src_repo_reltag, src_repo_type, src_repo_rev, src_repo_branch, spec_rev_form):

    # Check for valid revision form value
    spec_rev_forms = [ "postrel", "prerel" ]

    if spec_rev_form not in spec_rev_forms:
        raise ValueError("Invalid snapshot version form type")

    # Validate the VCS type
    vcs_types = [ "git", "hg", "svn" ]
    if src_repo_type not in vcs_types:
        raise ValueError("Invalid version control system type")

    # Generate a short hash for hg/git, reuse rev for subversion
    if src_repo_type == "git" or src_repo_type == "hg":
        repo_tag_shorthash = src_repo_rev[:7]
    else:
        repo_tag_shorthash = src_repo_rev

    # Generate the datetime stamp for the build
    pkg_build_datetime = datetime.datetime.now()

    pkg_build_date = "{0}{1}{2}".format(pkg_build_datetime.year,
                                        pkg_build_datetime.month,
                                        pkg_build_datetime.day)
    pkg_build_time = "{0}{1}".format(pkg_build_datetime.hour,
                                     pkg_build_datetime.minute)
    # pkg_build_time = "{0}{1}{2}".format(pkg_build_datetime.hour, pkg_build_datetime.minute, pkg_build_datetime.second)

    # Configure version form
    # 0: Version, 1: VCS, 2: Build date, 3: build time, 4: VCS revision

    if spec_rev_form == "postrel":
        spec_ver_form = "{0}+{1}{2}{3}.{4}.{5}"
    elif spec_rev_form == "prerel":
        spec_ver_form = "{0}~{1}{2}{3}.{4}.{5}"
    elif spec_rev_form == "prerel-upver":
        spec_ver_form = "{0}.0.1~{1}{2}{3}.{4}.{5}"
    else:
        spec_ver_form = "{0}.{1}{2}{3}.{4}.{5}"

    return spec_ver_form.format(src_repo_reltag, src_repo_type,
                                pkg_build_date, pkg_build_time,
                                repo_tag_shorthash,
                                __sanitize_label_for_version(src_repo_branch))
